/*
 * Copyright (c) 2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.zeitsync.par.VewZeitSyncParameter
 *
 * de.bsvrz.kex.tls.zeitsync.par.VewZeitSyncParameter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.kex.tls.zeitsync.par.VewZeitSyncParameter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.kex.tls.zeitsync.par.VewZeitSyncParameter; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.zeitsync.par;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

import java.io.*;
import java.util.*;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Klasse erstellt/ergnzt den Parametersatz fr die Zeitsynchronisation.
 * <p/>
 * Die Parameter-Eintrge werden aus einer ber den Aufrufparameter bergebenden
 * csv-Datei ausgelesen.
 * <p/>
 * ber den Aufrufparameter wird auch festgelegt, ob der Parametersatz berschrieben werden soll.
 * Falls ja, wird der Zeitsynchronisation-Parameter neu aus der csv-Datei erstellt.
 * Falls nein, wird der Zeitsynchronisation-Parameter mit Daten aus der csv-Datei ergnzt.
 * <p/>
 * Die Parameter-Eintrge werden nach Doppel-Eintrgen berprft (s.Methode {@link Parameter#equals(Object)}).
 * Falls die Eintrge gleich sind, wird nur letzter hinzugefgt (der letzte gewinnt).
 * Beim gleichen Eintrag aus DaV und aus der Datei gewinnt der Eintrag aus der Datei.
 * <p/>
 * Jeder Eintrag in der Datei besteht aus folgenden Informationen:
 * <li/> SO_PID: PID des SystemObjekts, entweder SteuerModul oder das bergeordnete Gert aus der TLS-Sicht;
 * <li/> DE_KANAL: De-Kanal 0 oder 255;
 * <li/> INTERVALL_STUNDEN Intervall der Zeitsynchronisation als Stundenangabe.
 * <li/> AKTIVIERT: ja oder nein.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 654 $ / $Date: 2012-11-02 12:01:23 +0100 (Fr, 02 Nov 2012) $ / ($Author: LG $)
 */
public class VewZeitSyncParameter implements StandardApplication, ClientSenderInterface {
    private static final String ASPEKT_SOLL    = "asp.parameterSoll";
    private static final String ASPEKT_VORGABE = "asp.parameterVorgabe";
    private static final String ATG            = "atg.paramTlsZeitSync";
    private static final String TRENN_LINIE    = "=======================================================================================\n";

    /** DebugLogger fr Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /** Legt fest, ob das Sendeerlaubnis fr den Parameter vorhanden ist. */
    private boolean         _sendeErlaubnis = false;
    private DataDescription _datenBeschreibung;

    /** Datenverteilerverbindung. */
    private ClientDavInterface _dav;

    /** Die Konfiguration des DaV. */
    private DataModel _davKonf;

    /** Parameter: csv-datei mit Daten fr den Parameter Zeit-Synchronisation. */
    private String _parDatei;

    /** Parameter: SystemObjekt-PID fr den Parameter Zeit-Synchronisation. */
    private String _parSoPid;

    /**
     * Parameter: Legt fest ob Parametersatz berschrieben werden soll
     * (ja - Parameter wird berschrieben, nein - Parameter wird ergnzt).
     */
    private boolean _parUeberschreiben;

    /** SystemObjekt fr den Parameter Zeit-Synchronisation. */
    private SystemObject _soZeitSync;

    //~ METHODEN ==============================================================

    /**
     * Debug-Info-Ausgabe der ParameterListe.
     *
     * @param titel Titel.
     * @param listePar ParameterListe.
     */
    private void ausgabe(String titel, List<Parameter> listePar) {
        StringBuilder txt = new StringBuilder(TRENN_LINIE);

        txt.append(titel).append(":\n");

        for (Parameter p : listePar) {
            txt.append(p).append("\n");
        }

        txt.append(TRENN_LINIE);
        debug.info(txt.toString());
    }

    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
        _sendeErlaubnis = (state == START_SENDING);

        String status = "unbekannter Status";

        if (state == START_SENDING) {
            status = "START_SENDING";
        } else if (state == STOP_SENDING) {
            status = "STOP_SENDING";
        } else if (state == STOP_SENDING_NO_RIGHTS) {
            status = "STOP_SENDING_NO_RIGHTS";
        } else if (state == STOP_SENDING_NOT_A_VALID_SUBSCRIPTION) {
            status = "STOP_SENDING_NOT_A_VALID_SUBSCRIPTION";
        }

        debug.fine(String.format("SO: %s%nDatenbeschreibung: %s%nStatus: %s%n", object, dataDescription, status));
    }

    /**
     * Ermittle die Liste aller SteuerModule des SystemObject.
     * Die Kinder-SteuerModule des Gerts werden rekursiv gesucht.
     *
     * @param so SystemObject.
     *
     * @return Liste aller SteuerModule des SystemObject.
     * ACHTUNG: die Liste wird nicht nach Doppel-Eontrge berprft.
     */
    private List<SystemObject> ermittleListeSM(SystemObject so) {
        List<SystemObject> listeSm = new ArrayList<SystemObject>();

        // Falls das SystemObject ein SM ist, wird nur es in die Liste eingefgt:
        if (so.getType().getPid().equals("typ.steuerModul")) {
            listeSm.add(so);
        }

        // sonst - ermittle alle Kinder-SteuerModule des Gerts rekursiv:
        else {

            // Hole die Menge der Anschlusspunktobjekte zu dem Gert:
            ObjectSet mengeAPGeraet = ((ConfigurationObject) so).getObjectSet("AnschlussPunkteGert");
            Data[]    arConfDaten;

            // Fr jeden Anschlusspunkt bestimme die Anschlusspunktkommunikationspartner
            for (SystemObject apkp : mengeAPGeraet.getElements()) {
                ConfigurationObject konfigurationsObjApkp    = (ConfigurationObject) apkp;
                ObjectSet           mengeAPKP                = konfigurationsObjApkp.getObjectSet("AnschlussPunkteKommunikationsPartner");  // Name der Menge, nicht PID !
                AttributeGroup      atgAnschlussPKommPartner = _davKonf.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");

                arConfDaten = _davKonf.getConfigurationData(mengeAPKP.getElements(), atgAnschlussPKommPartner);

                for (Data data : arConfDaten) {
                    listeSm.addAll(ermittleListeSM(data.getReferenceValue("KommunikationsPartner").getSystemObject()));
                }
            }
        }

        return listeSm;
    }

    @Override
    public void initialize(ClientDavInterface dav) throws Exception {

        // --------------------------------------------------------------
        // DaV-Zugriff:
        _dav     = dav;
        _davKonf = _dav.getDataModel();

        // --------------------------------------------------------------
        // Initialisiere ZeitSync-Objekt:
        _soZeitSync = _davKonf.getObject(_parSoPid);

        if (_soZeitSync == null) {
            debug.error(String.format("Das Objekte mit dem PID <%s> kann nicht ermittelt werden!. Bitte prfen Sie den Aufrufparameter <-soZeitsyncPid>. Programm wird beendet.", _parSoPid));

            throw new RuntimeException("Null-Objekt");
        }
    }

    //~ GET METHODEN ==========================================================

    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return true;
    }

    //~ METHODEN ==============================================================

    /**
     * Lade den Parameter ZeitSync aus Dav.
     *
     * @return Liste der aus DaV geladenen Parameter.
     */
    private List<Parameter> ladeParameterAusDav() {

        //
        debug.info(String.format("%s Lade  den Parametersatz aus DaV ...\n%s", TRENN_LINIE, TRENN_LINIE));

        //
        List<Parameter> listePar = new ArrayList<Parameter>();

        // --------------------------------------------------------------
        // Initialisiere DaV-Daten:
        AttributeGroup  atg               = _davKonf.getAttributeGroup(ATG);
        Aspect          asp               = _davKonf.getAspect(ASPEKT_SOLL);
        DataDescription datenBeschreibung = new DataDescription(atg, asp);

        // --------------------------------------------------------------
        ResultData daten = _dav.getData(_soZeitSync, datenBeschreibung, 0);

        // Lese den ZeitSync-Parameter aus DaV:
        if (daten.hasData()) {
            String  smPid;
            int     deKanal;
            long    zeitIntervallInMillis;
            boolean aktiviert;

            //
            Data.Array   ar = daten.getData().getArray("SmZeitSyncParameter");
            Data         data;
            SystemObject sm;

            for (int i = 0; i < ar.getLength(); i++) {

                // Ermittle Daten:
                data                  = ar.getItem(i);
                smPid                 = data.getReferenceValue("SteuerModulReferenz").getSystemObjectPid();
                deKanal               = data.getUnscaledValue("De0Oder255").intValue();
                zeitIntervallInMillis = data.getTimeValue("Intervall").getMillis();
                aktiviert             = (data.getTextValue("aktiviert").getText().equals("Ja"));

                // Ermittle SystemObject:
                sm = _davKonf.getObject(smPid);

                // Fge den Parameter in die Liste ein:
                if (sm != null) {
                    listePar.add(new Parameter(sm, deKanal, zeitIntervallInMillis, aktiviert));
                }

                // Warnung:
                else {
                    debug.warning(String.format("SteuerModul mit der PID <%s> ist nicht ermittelbar", smPid));
                }
            }
        }

        // Warnung:
        else {
            debug.warning(String.format("Parametersatz ZeitSync ist leer."));
        }

        // --------------------------------------------------------------
        // Ausgabe:
        ausgabe(String.format("Es wurden folgende <%d> Parameter-Eintrge aus DaV geladen", listePar.size()), listePar);

        //
        return listePar;
    }

    /**
     * Lese die Parameter-Eintrge aus der csv-Datei.
     *
     * @return Liste der aus der csv-Datei ausgelesenen Parameter.
     */
    private List<Parameter> leseParameterAusDerCsvDatei() {
        debug.info(String.format("%s Leser  den Parametersatz aus der Datei \n<%s>...\n%s", TRENN_LINIE, _parDatei, TRENN_LINIE));

        //
        List<Parameter> listePar = new ArrayList<Parameter>();
        boolean         fehler   = false;

        // --------------------------------------------------------------
        // Lese die csv-Datei:
        File datei = new File(_parDatei);

        //
        String spSoPid           = "SO_PID";
        String spDe              = "DE_KANAL";
        String spIntevallStunden = "INTERVALL_STUNDEN";
        String spAktiviert       = "AKTIVIERT";

        //
        String       soPid;
        SystemObject so;
        int          deKanal;
        long         intervall;
        boolean      aktiviert;
        Parameter    par;

        //
        CsvLeser csvDatei;
        int      zeilenNr = 1;

        try {

            // Datei einlesen...
            csvDatei = new CsvLeser(new FileReader(datei), ';');

            Map<String, String> zeile;

            // --------------------------------------------------------------
            // fr jede Zeile:
            while (csvDatei.hasNochMehrZeilen()) {
                zeile = csvDatei.getNaechsteZeile();

                // --------------------------------------------------------------
                // lese und prfe Daten der Zeile:
                soPid   = zeile.get(spSoPid).trim();
                deKanal = Integer.valueOf(zeile.get(spDe).trim());

                if ((deKanal != 0) && (deKanal != 255)) {
                    debug.error(String.format("Unbekannter deKanal <%d> in der Zeile <%d> (darf 0 oder 255 sein)", deKanal, zeilenNr));
                    fehler = true;

                    break;
                }

                intervall = Integer.valueOf(zeile.get(spIntevallStunden).trim()) * 60 * 60 * 1000;
                aktiviert = (zeile.get(spAktiviert).trim().equals("ja"));
                so        = _davKonf.getObject(soPid);

                if (so == null) {
                    debug.error(String.format("Unbekanntes SystemObject <%s> in der Zeile <%d>", soPid, zeilenNr));
                    fehler = true;

                    break;
                }

                // --------------------------------------------------------------
                // Ermittle alle SteuerModule fr das SystemObject:
                List<SystemObject> listeSm = ermittleListeSM(so);

                if (listeSm.isEmpty()) {
                    debug.error(String.format("Keine SteuerModule sind fr das SystemObject <%s> aus der Zeile <%d> ermittelbar", soPid, zeilenNr));
                    fehler = true;

                    break;
                }

                // --------------------------------------------------------------
                // Einfge  Parameter in die Liste:
                for (SystemObject sm : listeSm) {
                    par = new Parameter(sm, deKanal, intervall, aktiviert);
                    listePar.add(par);
                }

                //
                zeilenNr++;
            }

            csvDatei.close();
        }
        catch (FileNotFoundException e) {
            debug.error(String.format("Datei <%s> kann nicht gefunden werden!", datei), e);
            fehler = true;
        }
        catch (NumberFormatException e) {
            debug.error(String.format("Fehler beim Einlesen der Datei <%s> in der Zeile <%d>.", datei, zeilenNr), e);
            fehler = true;
        }
        catch (Exception e) {
            debug.error(String.format("Fehler beim Einlesen der Datei <%s> in der Zeile <%d>.", datei, zeilenNr), e);
            fehler = true;
        }

        // --------------------------------------------------------------
        // Falls _Fehler aufgetreten sind:
        if (fehler) {
            return new ArrayList<Parameter>();
        }

        // --------------------------------------------------------------
        // Ausgabe:
        ausgabe(String.format("Es wurden folgende <%d> Parameter-Eintrge aus der Datei gelesen", listePar.size()), listePar);

        // --------------------------------------------------------------
        return listePar;
    }

    /**
     * Main-Methode.
     *
     * @param args Parameter.
     */
    public static void main(String[] args) {

        //
        VewZeitSyncParameter vewPar = new VewZeitSyncParameter();

        StandardApplicationRunner.run(vewPar, args);

        // Debug erst hier initialisieren, da erst ab hier verfgbar !!
        debug = Debug.getLogger();

        // Sende den Parametersatz:
        vewPar.sendeZeitSyncParameter();

        // Ende:
        System.exit(0);
    }

    @Override
    public void parseArguments(ArgumentList argumentList) throws Exception {
        String par;

        // --------------------------------------------------------------
        // Hole den Parameter 'SystemObjekt PID fr die Zeit-Synchronisation':
        par = "-soZeitsyncPid";

        try {
            _parSoPid = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte die SystemObjekt-PID fr die Zeit-Synchronisation als Parameter <%s> bergeben!", par), e);

            throw new RuntimeException(e);
        }

        // --------------------------------------------------------------
        // Hole den Parameter 'csv-Datei mit Parameter-Daten':
        par = "-csvDatei";

        try {
            _parDatei = argumentList.fetchArgument(par).asNonEmptyString();
        }
        catch (Exception e) {
            debug.error(String.format("Bitte die csv-Datei mit Parameter-Daten als Parameter <%s> bergeben!", par), e);

            throw new RuntimeException(e);
        }

        // --------------------------------------------------------------
        // Hole den Parameter 'Parameter berschreiben'
        // (ja - Parameter wird berschrieben, nein - Parameter wird ergnzt):
        par = "-parameterUeberschreiben";

        try {
            String parUeberschreiben = argumentList.fetchArgument(par).asNonEmptyString().toLowerCase();

            if (parUeberschreiben.equals("ja")) {
                _parUeberschreiben = true;
            } else if (parUeberschreiben.equals("nein")) {
                _parUeberschreiben = false;
            } else {
                debug.error(String.format("Unzulssiger Wert fr den Parameter <%s=%s> (darf nur <ja> oder <nein> sein)!", par, parUeberschreiben));

                throw new RuntimeException();
            }
        }
        catch (Exception e) {
            debug.error(String.format("Bitte den Parameter <%s> mit dem Wert <ja> oder <nein> bergeben!", par), e);

            throw new RuntimeException(e);
        }
    }

    /**
     * Erstelle den Zeitsynchronitation-Parameter aus den DaV-Daten und der csv-Datei
     * und sende den an DaV.
     */
    private void sendeZeitSyncParameter() {

        // --------------------------------------------------------------
        // Lade Parameter aus DaV, falls den Parameter ergnszt werden soll:
        // --------------------------------------------------------------
        List<Parameter> listeParGelesen;

        if (!_parUeberschreiben) {
            listeParGelesen = ladeParameterAusDav();
        } else {
            listeParGelesen = new ArrayList<Parameter>();
        }

        // --------------------------------------------------------------
        // Erstelle DaV-Parameter aus csv-Datei und fge die in der Liste:
        // --------------------------------------------------------------
        {
            List<Parameter> listeParAusDatei = leseParameterAusDerCsvDatei();

            if (listeParAusDatei.isEmpty()) {
                debug.error("Die Liste der zu erstellten Parameter ist leer!");

                throw new RuntimeException();
            }

            // Fge die erstellten Parameter in die Liste:
            listeParGelesen.addAll(listeParAusDatei);
        }

        // --------------------------------------------------------------
        // Erstelle die neue Liste der Parameter aus der Eintrgen aus DaV und aus der Datei.
        //
        // Falls die Eintrge gleich sind (s. Methode Parameter.equal()),
        // wird nur letzter hinzugefgt (der letzte gewinnt).
        //
        // Beim gleichen Eintrag aus DaV und aus der Datei
        // gewinnt der Eintrag aus der Datei.
        // --------------------------------------------------------------
        debug.info(String.format("%s Erstelle neuen Parametersatz ...\n%s", TRENN_LINIE, TRENN_LINIE));

        //
        List<Parameter> listeParNeu                 = new ArrayList<Parameter>();
        List<Parameter> listeParNichtBercksichtigt = new ArrayList<Parameter>();

        // Betrachte in der umgekehrten Reihenfolge um die Prioritt
        // der Eintrge aus der Datei vor den Eintrge aus DaV und
        // die Prioritt des letzten Eintrags zu schafen:
        for (int i = listeParGelesen.size() - 1; i >= 0; i--) {
            Parameter p = listeParGelesen.get(i);

            if (listeParNeu.contains(p)) {
                listeParNichtBercksichtigt.add(0, p);
            } else {
                listeParNeu.add(p);
            }
        }

        // Sortiere die Liste:
        Collections.sort(listeParNeu, new Comparator<Parameter>() {
            @Override
            public int compare(Parameter p1, Parameter p2) {
                return p1._sm.getPidOrNameOrId().compareTo(p2._sm.getPidOrNameOrId());
            }
        });

        // Ausgabe:
        if (!listeParNichtBercksichtigt.isEmpty()) {
            ausgabe(String.format("Folgende <%d> Parameter-Eintrge wurden nicht bercksichtigt (da sie mehrmals vorhanden sind)", listeParNichtBercksichtigt.size()), listeParNichtBercksichtigt);
        }

        ausgabe(String.format("Es sind folgende neue <%d> Parameter-Eintrge erstellt", listeParNeu.size()), listeParNeu);

        // --------------------------------------------------------------
        // Initialisiere DaV-Daten:
        // --------------------------------------------------------------
        AttributeGroup atg = _davKonf.getAttributeGroup(ATG);
        Aspect         asp = _davKonf.getAspect(ASPEKT_VORGABE);

        _datenBeschreibung = new DataDescription(atg, asp);

        // --------------------------------------------------------------
        // Anmeldung als Sender:
        // --------------------------------------------------------------
        try {
            _dav.subscribeSender(this, _soZeitSync, _datenBeschreibung, SenderRole.sender());
        }
        catch (OneSubscriptionPerSendData e) {
            debug.error(String.format("Datenidentifikation ist bereits angemeldet. Eventuell Empfnger nicht vorhanden: <%s>, <%s>. Programm wird beendet.", _soZeitSync, _datenBeschreibung), e);

            throw new RuntimeException(e);
        }
        catch (Exception e) {
            debug.error(String.format("Fehler bei der Anmeldung der Daten als Sender: <%s>, <%s>. Programm wird beendet.", _soZeitSync, _datenBeschreibung), e);

            throw new RuntimeException(e);
        }

        // -------------------------------------------------
        // Warte auf SendeErlaubnis:
        // Ein paar mal versuchen, ob Sendeerlaubnis bereits vorliegt...
        // -------------------------------------------------
        int i = 0;

        while (i < 10) {
            if (_sendeErlaubnis) {
                break;
            } else {

                // Pause:
                try {
                    Thread.sleep(500);
                }
                catch (Exception ignored) {}
            }

            i++;
        }

        if (!_sendeErlaubnis) {
            debug.error(String.format("Sendeerlaubnis ist nicht vorhanden: <%s>, <%s>", _soZeitSync, _datenBeschreibung));

            throw new RuntimeException("Sendeerlaubnis ist nicht vorhanden");
        }

        // -------------------------------------------------
        // Erstelle DaV-Daten aus der Liste der Parameter:
        // -------------------------------------------------
        Data daten = _dav.createData(atg);

        // Urlasser-Daten:
        String ursache;

        if (_parUeberschreiben) {
            ursache = "Neue Anlage des Parametersatzes";
        } else {
            ursache = "Ergnzung des Parametersatzes";
        }

        daten.getItem("Urlasser").getTextValue("Ursache").setText(ursache);
        daten.getItem("Urlasser").getTextValue("Veranlasser").setText("Tool zur Verwaltung des ZeitSyncParameters <de.bsvrz.kex.tls.zeitsync.par>");

        // SmZeitSyncParameter
        Data.Array ar = daten.getArray("SmZeitSyncParameter");

        ar.setLength(listeParNeu.size());

        Parameter par;
        Data      data;

        for (i = 0; i < listeParNeu.size(); i++) {
            par  = listeParNeu.get(i);
            data = ar.getItem(i);
            data.getReferenceValue("SteuerModulReferenz").setSystemObject(par._sm);
            data.getUnscaledValue("De0Oder255").set(par._de0Oder255);
            data.getTimeValue("Intervall").setMillis(par._intervall);
            data.getTextValue("aktiviert").setText(par._aktiviert ? "Ja" : "Nein");
        }

        ResultData ergebnis = new ResultData(_soZeitSync, _datenBeschreibung, System.currentTimeMillis(), daten, false);

        // -------------------------------------------------
        // Sende DaV-Daten:
        // -------------------------------------------------
        try {
            _dav.sendData(ergebnis);

            // Ausgabe:
            debug.info(String.format("%s Es sind <%d> Eintrge fr den Zeitsynchronisation-Parameter erfolgreich gesendet.\n%s", TRENN_LINIE, listeParNeu.size(), TRENN_LINIE));
        }
        catch (Exception e) {
            debug.error(String.format("Fehler beim Senden der Daten an DaV: <%s>, <%s>", _soZeitSync, _datenBeschreibung), e);

            throw new RuntimeException(e);
        }
    }

    //~ INNERE KLASSEN ========================================================

    /**
     * Daten fr den Zeitsynchronitation-Parameter.
     */
    private class Parameter {
        private boolean      _aktiviert;
        private int          _de0Oder255;
        private long         _intervall;
        private SystemObject _sm;

        //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==========

        /**
         * Erstellt eine Instanz der Klasse Parameter.
         *
         * @param sm SteuerModul.
         * @param de0Oder255 DE-Kanal (0 oder 255).
         * @param intervall ZeitIntervall in Millis zwischen den Ausfhren der Auftrge.
         * @param aktiviert Legt fest, ob der Auftragt aktiviert ist (Wert == true).
         */
        private Parameter(SystemObject sm, int de0Oder255, long intervall, boolean aktiviert) {
            _sm         = sm;
            _de0Oder255 = de0Oder255;
            _intervall  = intervall;
            _aktiviert  = aktiviert;
        }

        //~ METHODEN ==========================================================

        /**
         * Stellt fest, ob die Parameter gleich sind.
         *
         * @param obj 2.Parameter.
         * @return true, falls SteuerModul und DeKanal gleich sind,
         * false sonst.
         */
        @Override
        public boolean equals(Object obj) {

            // Parameter sind gleich, falls SteuerModul und DeKanal gleich sind:
            if (obj instanceof Parameter) {
                Parameter par = (Parameter) obj;

                return ((_sm.equals(par._sm)) && (_de0Oder255 == par._de0Oder255));
            }

            return false;
        }

        //~ GET METHODEN ======================================================

        /**
         * Stellt die Zeitdauer als String dar.
         *
         * @param zeitDauer Zeitdauer, die als String dargestellt werden soll.
         *
         * @return Zeitdauer als String "hh:mm:ss".
         */
        private String getZeitdauerAlsString(long zeitDauer) {
            long  zeitHr  = zeitDauer / (1000 * 60 * 60);
            long  zeitMin = (zeitDauer - zeitHr * 60 * 60 * 1000) / (1000 * 60);
            float zeitSek = (zeitDauer - zeitHr * 60 * 60 * 1000 - zeitMin * 60 * 1000) / 1000f;

            return String.format("%02d:%02d:%02d", zeitHr, zeitMin, Math.round(zeitSek));
        }

        //~ METHODEN ==========================================================

        @Override
        public String toString() {
            return String.format("%-55s  %3d  %s  %s", _sm.getPidOrNameOrId(), _de0Oder255, getZeitdauerAlsString(_intervall), _aktiviert);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 02.11.12
